Imports Windows.Graphics.Imaging
Imports Windows.Storage

''' <summary>
''' Generates a thumbnnail from an image
''' </summary>
Public NotInheritable Class ThumbnailMaker
    ' URL format for referencing local storage
    Private Const URI_LOCAL As String = "ms-appdata:///Local/{0}"

    ''' <summary>
    ''' Creates a thumbnail image in local storage based on the source file, then returns the URI to the thumbnail
    ''' </summary>
    ''' <param name="file">The source image file to generate the thumbnail from</param>
    ''' <returns>The URI to the generated thumbnail in local storage</returns>
    Public Function GenerateThumbnailAsync(ByVal file As IStorageFile) As IAsyncOperation(Of Uri)
        Return GenerateThumbnail(file).AsAsyncOperation
    End Function

    Private Shared Async Function GenerateThumbnail(ByVal file As IStorageFile) As Task(Of Uri)
        Using fileStream = Await file.OpenReadAsync
            ' decode the file using the built-in image decoder
            Dim decoder = Await BitmapDecoder.CreateAsync(fileStream)

            ' create the output file for the thumbnail
            Dim thumbFile = Await ApplicationData.Current.LocalFolder.CreateFileAsync(String.Format("thumbnail{0}", file.FileType), CreationCollisionOption.GenerateUniqueName)

            ' create a stream for the output file
            Using outputStream = Await thumbFile.OpenAsync(FileAccessMode.ReadWrite)
                ' create an encoder from the existing decoder and set the scaled height
                ' and width 
                Dim encoder = Await BitmapEncoder.CreateForTranscodingAsync(outputStream, decoder)
                encoder.BitmapTransform.ScaledHeight = 100
                encoder.BitmapTransform.ScaledWidth = 100
                Await encoder.FlushAsync
            End Using

            ' create the URL
            Dim storageUrl = String.Format(URI_LOCAL, thumbFile.Name)

            ' return it
            Return New Uri(storageUrl, UriKind.Absolute)
        End Using
    End Function
End Class

